@@ -1,7 +1,4 @@
-"""A module for creating relative permeability input curves for
-Eclipse and Nexus.
-"""
-
+"""pyscal"""
 from __future__ import division, absolute_import
 from __future__ import print_function
 
@@ -10,6 +7,7 @@ from .utils import interpolator  # noqa
 from .wateroil import WaterOil  # noqa
 from .wateroilgas import WaterOilGas  # noqa
 from .gasoil import GasOil  # noqa
+from .gaswater import GasWater  # noqa
 from .scalrecommendation import SCALrecommendation  # noqa
 from .pyscallist import PyscalList  # noqa
 from .factory import PyscalFactory  # noqa
