@@ -8,6 +8,8 @@ import six
 import numpy as np
 import pandas as pd
 
+from scipy.interpolate import PchipInterpolator
+
 import pyscal
 from pyscal import utils
 from pyscal.constants import EPSILON as epsilon
@@ -171,7 +173,7 @@ class GasOil(object):
             self.table["sgn"] = (self.table["sg"] - sgcr) / (1 - swl - sgcr)
 
         # Normalized oil saturation should be 0 at 1-sorg, and 1 at swl+sgcr
-        self.table["son"] = (self.table["sl"] - sorg - swl) / (1 - sorg - swl - sgcr)
+        self.table["son"] = (self.table["sl"] - sorg - swl) / (1 - sorg - swl)
         self.sgcomment = "-- swirr=%g, sgcr=%g, swl=%g, sorg=%g, krgendanchor=%s\n" % (
             self.swirr,
             self.sgcr,
@@ -183,7 +185,7 @@ class GasOil(object):
         self.krogcomment = ""
         self.pccomment = ""
 
-        logger.info(
+        logger.debug(
             "Initialized GasOil with %s saturation points", str(len(self.table))
         )
 
@@ -226,8 +228,6 @@ class GasOil(object):
         Calling function is responsible for checking if any data was
         actually added to the table.
         """
-        from scipy.interpolate import PchipInterpolator
-
         # Avoid having to deal with multi-indices:
         if len(dframe.index.names) > 1:
             logger.warning(
@@ -368,40 +368,31 @@ class GasOil(object):
             ] = tmp.loc[tmp.sg >= (1 - (self.sorg + self.swl + epsilon)), "krg"]
         else:
             self.table.loc[self.table.sg > (1 - (self.swl + epsilon)), "krg"] = krgend
-            if krgmax and krgmax < 1.0:
+            if krgmax and krgmax < 1.0 and self.sorg > 0:
                 # Only warn if something else than default is in use
                 logger.warning("krgmax ignored when not anchoring to sorg")
 
-    def set_endpoints_linearpart_krog(self, kroend, kromax):
+    def set_endpoints_linearpart_krog(self, kroend, kromax=None):
         """Set linear parts of krog outside endpoints.
 
         Zero for sg above 1 - sorg - swl.
 
-        Linear from kromax to kroend from sg in [0, sgcr]
-
         This function is used by add_corey/LET_oil(), and perhaps by other
         utility functions. It should not be necessary for end-users.
 
         Args:
-            kroend (float): krog at sg=sgcr
-            kromax (float): krog at Sg=0. Default 1.
-
+            kroend (float): krog at sg=0
         """
+        if kromax is not None:
+            logger.error("kromax is DEPRECATED, ignored")
+
         # Special handling of the part close to sg=1, set to zero.
         self.table.loc[
             self.table["sg"] > 1 - self.sorg - self.swl - epsilon, "krog"
         ] = 0
 
-        # Set kromax at sg=0, but only if sgcr is sufficiently larger than 0.
-        if self.sgcr > 1.0 / SWINTEGERS:
-            if not kromax:
-                kromax = 1
-            self.table.loc[self.table["sg"] < 1.0 / SWINTEGERS, "krog"] = kromax
-        else:
-            if kromax and kromax < 1.0:
-                # Only warn for non-defaulted values
-                logger.warning("kromax ignored when sgcr is close to zero")
-            self.table.loc[self.table["sg"] < 1.0 / SWINTEGERS, "krog"] = kroend
+        # Floating point issues can cause a slight overshoot at sg=0:
+        self.table.loc[self.table["krog"] > kroend, "krog"] = kroend
 
     def add_corey_gas(self, ng=2, krgend=1, krgmax=None):
         """ Add krg data through the Corey parametrization
@@ -440,33 +431,24 @@ class GasOil(object):
 
         All values above 1 - sorg - swl are set to zero.
 
-        kromax is ignored if sgcr is close to zero
-
         Arguments:
             nog (float): Corey exponent for oil
             kroend (float): Value for krog at normalized oil saturation 1
-            kromax (float): Value for krog at gas saturation 0.
 
         Returns:
             None (modifies internal class state)
         """
         assert epsilon < nog < MAX_EXPONENT
-        if kromax:
-            assert 0 < kroend <= kromax <= 1.0
-        else:
-            assert 0 < kroend <= 1.0
+        assert 0 < kroend <= 1.0
+
+        if kromax is not None:
+            logger.error("kromax is DEPRECATED, ignored")
 
         self.table["krog"] = kroend * self.table.son ** nog
 
-        self.set_endpoints_linearpart_krog(kroend, kromax)
+        self.set_endpoints_linearpart_krog(kroend)
 
-        if not kromax:
-            kromax = 1
-        self.krogcomment = "-- Corey krog, nog=%g, kroend=%g, kromax=%g\n" % (
-            nog,
-            kroend,
-            kromax,
-        )
+        self.krogcomment = "-- Corey krog, nog=%g, kroend=%g\n" % (nog, kroend,)
 
     def add_LET_gas(self, l=2, e=2, t=2, krgend=1, krgmax=None):
         """
@@ -488,6 +470,9 @@ class GasOil(object):
         Returns:
             None (modifies internal state)
         """
+        # Similar code in wateroil.add_LET_water, but readability
+        # is better by having them separate
+        # pylint: disable=duplicate-code
         assert epsilon < l < MAX_EXPONENT
         assert epsilon < e < MAX_EXPONENT
         assert epsilon < t < MAX_EXPONENT
@@ -524,22 +509,19 @@ class GasOil(object):
 
         All values where sg > 1 - sorg - swl are set to zero.
 
-        kromax is ignored if sgcr is close to zero
-
         Arguments:
             l (float): L parameter
             e (float): E parameter
             t (float): T parameter
             kroend (float): The value at gas saturation sgcr
-            kromax (float): The value at gas saturation equal to 0.
         """
         assert epsilon < l < MAX_EXPONENT
         assert epsilon < e < MAX_EXPONENT
         assert epsilon < t < MAX_EXPONENT
-        if kromax:
-            assert 0 < kroend <= kromax <= 1.0
-        else:
-            assert 0 < kroend <= 1.0
+        assert 0 < kroend <= 1.0
+
+        if kromax is not None:
+            logger.error("kromax is DEPRECATED, ignored")
 
         # LET shape for the interval [sgcr, 1 - swl - sorg]
         self.table["krog"] = (
@@ -550,16 +532,13 @@ class GasOil(object):
         # This equation is undefined for t a float and son=1, set explicitly:
         self.table.loc[np.isclose(self.table["son"], 1.0), "krog"] = kroend
 
-        self.set_endpoints_linearpart_krog(kroend, kromax)
+        self.set_endpoints_linearpart_krog(kroend)
 
-        if not kromax:
-            kromax = 1
-        self.krogcomment = "-- LET krog, l=%g, e=%g, t=%g, kroend=%g, kromax=%g\n" % (
+        self.krogcomment = "-- LET krog, l=%g, e=%g, t=%g, kroend=%g\n" % (
             l,
             e,
             t,
             kroend,
-            kromax,
         )
 
     def estimate_sorg(self):
@@ -621,26 +600,11 @@ class GasOil(object):
         when initializing the saturation range (it uses linear
         interpolation to solve for the zero)
 
-        Warning: Code duplication from WaterOil, with
-        column names changed only
+        Returns:
+            float: the gas saturation where krg == krog, for relperm
+                linearly interpolated in gas saturation.
         """
-
-        # Make a copy for calculations
-        tmp = pd.DataFrame(self.table[["sg", "krg", "krog"]])
-        tmp.loc[:, "krgminuskrog"] = tmp["krg"] - tmp["krog"]
-
-        # Add a zero value for the difference column, and interpolate
-        # the sw column to the zero value
-        zerodf = pd.DataFrame(index=[len(tmp)], data={"krgminuskrog": 0.0})
-        tmp = pd.concat([tmp, zerodf], sort=True)
-
-        tmp.set_index("krgminuskrog", inplace=True)
-        if tmp.index.isnull().any():
-            logger.warning("Could not compute crosspoint. Bug?")
-            return -1
-        tmp.interpolate(method="slinear", inplace=True)
-
-        return tmp[np.isclose(tmp.index, 0.0)].sg.values[0]
+        return utils.crosspoint(self.table, "sg", "krg", "krog")
 
     def selfcheck(self, mode="SGOF"):
         """Check validities of the data in the table.
@@ -699,7 +663,7 @@ class GasOil(object):
                 error = True
         if error:
             return False
-        logger.info("GasOil object is checked to be valid")
+        logger.debug("GasOil object is checked to be valid")
         return True
 
     def SGOF(self, header=True, dataincommentrow=True):
@@ -736,7 +700,7 @@ class GasOil(object):
             string += self.krgcomment
             string += self.krogcomment
             if not self.fast:
-                string += "-- krg = krog @ sw=%1.5f\n" % self.crosspoint()
+                string += "-- krg = krog @ sg=%1.5f\n" % self.crosspoint()
             string += self.pccomment
         width = 10
         string += (
@@ -827,7 +791,7 @@ class GasOil(object):
             string += self.sgcomment
             string += self.krgcomment
             string += self.krogcomment
-            string += "-- krg = krog @ sw=%1.5f\n" % self.crosspoint()
+            string += "-- krg = krog @ sg=%1.5f\n" % self.crosspoint()
             string += self.pccomment
         width = 10
         string += (
@@ -844,7 +808,9 @@ class GasOil(object):
         string += "/\n"
         return string
 
-    def SGFN(self, header=True, dataincommentrow=True):
+    def SGFN(
+        self, header=True, dataincommentrow=True, sgcomment=None, crosspointcomment=None
+    ):
         """
         Produce SGFN input for Eclipse reservoir simulator.
 
@@ -855,12 +821,18 @@ class GasOil(object):
         as Eclipse comments.
 
         Args:
-            header: boolean for whether the SGOF string should be emitted.
+            header: boolean for whether the SGFN string should be emitted.
                 If you have multiple satnums, you should have True only
-                for the first (or False for all, and emit the SGOF yourself).
+                for the first (or False for all, and emit the SGFN yourself).
                 Defaults to True.
             dataincommentrow: boolean for wheter metadata should be printed,
                 defaults to True.
+            sgcomment (str): Provide the string to include in the comment
+                section for describing the saturation endpoints. Used
+                by GasWater.
+            crosspointcomment (str): String to be used for crosspoint comment
+                string, overrides what this object can provide. Used by GasWater.
+                If None, it will be computed, use empty string to avoid.
         """
         string = ""
         if "pc" not in self.table.columns:
@@ -871,10 +843,16 @@ class GasOil(object):
         string += utils.comment_formatter(self.tag)
         string += "-- pyscal: " + str(pyscal.__version__) + "\n"
         if dataincommentrow:
-            string += self.sgcomment
+            if sgcomment is not None:
+                string += sgcomment
+            else:
+                string += self.sgcomment
             string += self.krgcomment
-            if "krog" in self.table.columns:
-                string += "-- krg = krog @ sw=%1.5f\n" % self.crosspoint()
+            if crosspointcomment is None:
+                if "krog" in self.table.columns:
+                    string += "-- krg = krog @ sg=%1.5f\n" % self.crosspoint()
+            else:
+                string += crosspointcomment
             string += self.pccomment
         width = 10
         string += (
