@@ -5,7 +5,7 @@ import argparse
 
 import logging
 
-from pyscal import PyscalFactory
+from pyscal import WaterOilGas, GasWater, PyscalFactory
 
 from pyscal import __version__
 
@@ -20,7 +20,7 @@ column called TAG or COMMENT. Column headers are case insensitive.
 
 Saturation endpoints are put in columns 'swirr', 'swl', 'swcr', 'sorw',
 'sgcr' and 'sorg'. Relative permeability endpoints are put in columns
-'krwend', 'krwmax', 'krowend', 'krogend', 'kromax', 'krgend' and 'krgmax'.
+'krwend', 'krwmax', 'krowend', 'krogend', 'krgend' and 'krgmax'.
 These columns are optional and are defaulted to 0 or 1.
 
 Corey or LET parametrization are based on presence of the columns
@@ -37,7 +37,7 @@ For SCAL recommendations, there should be exactly three rows for each SATNUM,
 tagged with the strings 'low', 'base' and 'high' in the column 'CASE'
 
 When interpolating in a SCAL recommendation, 'int_param_wo' is the main parameter
-that is used for both water-oil and gas-oil, and for all SATNUMs if nothing
+that is used for water-oil, gas-oil and gas-water, and for all SATNUMs if nothing
 more is provided. Provide int_param_go in addition if separate interpolation
 for WaterOil and GasOil is needed, and specify multiple floats pr. parameter
 if individual interpolation for each SATNUM is needed.
@@ -71,6 +71,9 @@ def get_parser():
         action="store_true",
         help="Print informational messages while processing input",
     )
+    parser.add_argument(
+        "--debug", action="store_true", help="Print debug information",
+    )
     parser.add_argument(
         "-o",
         "--output",
@@ -93,6 +96,7 @@ def get_parser():
             "low, base and high for each SATNUM. "
             "Either one number used for all SATNUM, or a sequence of "
             "length equal to the max SATNUM. Numbers between -1 and 1. "
+            "Also used for GasWater."
         ),
     )
     parser.add_argument(
@@ -106,7 +110,7 @@ def get_parser():
             "Either one number used for all SATNUM, or a sequence "
             "of length equal to the max SATNUM. Numbers between -1 and 1. "
             "If not provided, the water-oil interpolation parameter will be used "
-            "as default."
+            "as default. Do not use for GasWater."
         ),
     )
     parser.add_argument(
@@ -127,7 +131,8 @@ def get_parser():
         default=False,
         help=(
             "Output family 2 keywords, SWFN, SGFN and SOF3/SOF2. "
-            "Family 1 (SWOF + SGOF) is written if this is not set"
+            "Family 1 (SWOF + SGOF) is written if this is not set.  "
+            "Implicit for gas-water input."
         ),
     )
     return parser
@@ -144,6 +149,7 @@ def main():
         pyscal_main(
             parametertable=args.parametertable,
             verbose=args.verbose,
+            debug=args.debug,
             output=args.output,
             delta_s=args.delta_s,
             int_param_wo=args.int_param_wo,
@@ -160,6 +166,7 @@ def main():
 def pyscal_main(
     parametertable,
     verbose=False,
+    debug=False,
     output="relperm.inc",
     delta_s=None,
     int_param_wo=None,
@@ -183,17 +190,25 @@ def pyscal_main(
         slgof (bool): Use SLGOF
         family2 (bool): Dump family 2 keywords
     """
-    if verbose:
-        # Fixme: Logging level is not inherited in called modules.
-        logger.setLevel(logging.INFO)
 
-    if sheet_name:
-        logger.info("Loading data from %s and sheetname %s", parametertable, sheet_name)
-    else:
-        logger.info("Loading data from %s", parametertable)
+    def set_logger_levels(loglevel):
+        """Set log levels for all modules imported by this script"""
+        logger.setLevel(loglevel)
+        logging.getLogger("pyscal.factory").setLevel(loglevel)
+        logging.getLogger("pyscal.wateroil").setLevel(loglevel)
+        logging.getLogger("pyscal.wateroilgas").setLevel(loglevel)
+        logging.getLogger("pyscal.gasoil").setLevel(loglevel)
+        logging.getLogger("pyscal.utils").setLevel(loglevel)
+        logging.getLogger("pyscal.pyscallist").setLevel(loglevel)
+
+    if verbose:
+        set_logger_levels(logging.INFO)
+    if debug:
+        set_logger_levels(logging.DEBUG)
+
     scalinput_df = PyscalFactory.load_relperm_df(parametertable, sheet_name=sheet_name)
 
-    logger.info("Input data:\n%s", scalinput_df.to_string(index=False))
+    logger.debug("Input data:\n%s", scalinput_df.to_string(index=False))
 
     if int_param_go is not None and int_param_wo is None:
         logger.error("Don't use int_param_go alone, only int_param_wo")
@@ -209,12 +224,18 @@ def pyscal_main(
         scalrec_list = PyscalFactory.create_scal_recommendation_list(
             scalinput_df, h=delta_s
         )
-        logger.info(
-            "Interpolating, wateroil=%s, gasoil=%s",
-            str(int_param_wo),
-            str(int_param_go),
-        )
-        wog_list = scalrec_list.interpolate(int_param_wo, int_param_go, h=delta_s)
+        if scalrec_list[1].type == WaterOilGas:
+            logger.info(
+                "Interpolating, wateroil=%s, gasoil=%s",
+                str(int_param_wo),
+                str(int_param_go),
+            )
+            wog_list = scalrec_list.interpolate(int_param_wo, int_param_go, h=delta_s)
+        elif scalrec_list[1].type == GasWater:
+            logger.info(
+                "Interpolating, gaswater=%s", str(int_param_wo),
+            )
+            wog_list = scalrec_list.interpolate(int_param_wo, None, h=delta_s)
     else:
         wog_list = PyscalFactory.create_pyscal_list(
             scalinput_df, h=delta_s
@@ -227,20 +248,21 @@ def pyscal_main(
             "Interpolation parameter provided but no CASE column in input data"
         )
         raise ValueError
-    if not family2:
+
+    if family2 or wog_list.pyscaltype == GasWater:
+        logger.info("Generating family 2 keywords.")
+        if output == "-":
+            print(wog_list.dump_family_2())
+        else:
+            wog_list.dump_family_2(filename=output)
+            print("Written to " + output)
+    else:
         logger.info("Generating family 1 keywords.")
         if output == "-":
             print(wog_list.dump_family_1(slgof=slgof))
         else:
             wog_list.dump_family_1(filename=output, slgof=slgof)
             print("Written to " + output)
-    else:
-        logger.info("Generating family 2 keywords")
-        if output == "-":
-            print(wog_list.dump_family_2())
-        else:
-            wog_list.dump_family_2(filename=output)
-            print("Written to " + output)
 
 
 if __name__ == "__main__":
