@@ -10,6 +10,8 @@ import six
 import numpy as np
 import pandas as pd
 
+from scipy.interpolate import PchipInterpolator, interp1d
+
 import pyscal
 from pyscal.constants import EPSILON as epsilon
 from pyscal.constants import SWINTEGERS, MAX_EXPONENT
@@ -58,10 +60,22 @@ class WaterOil(object):
     """
 
     def __init__(
-        self, swirr=0.0, swl=0.0, swcr=0.0, sorw=0.0, h=0.01, tag="", fast=False
+        self,
+        swirr=0.0,
+        swl=0.0,
+        swcr=0.0,
+        sorw=0.0,
+        h=0.01,
+        tag="",
+        fast=False,
+        _sgcr=None,
     ):
         """Sets up the saturation range. Swirr is only relevant
-        for the capillary pressure, not for relperm data."""
+        for the capillary pressure, not for relperm data.
+
+        _sgcr is only to be used by the GasWater object.
+        """
+
         assert -epsilon < swirr < 1.0 + epsilon
         assert -epsilon < swl < 1.0 + epsilon
         assert -epsilon < swcr < 1.0 + epsilon
@@ -81,6 +95,9 @@ class WaterOil(object):
         else:
             self.h = h
 
+        if _sgcr is not None:
+            self.sgcr = _sgcr
+
         if not isinstance(tag, six.string_types):
             tag = ""
         self.swirr = swirr
@@ -125,26 +142,37 @@ class WaterOil(object):
         self.table = self.table[["sw"]]  # Drop the swint column
 
         # Normalize for krw:
-        self.table["swn"] = (self.table.sw - self.swcr) / (1 - self.swcr - self.sorw)
+        self.table["swn"] = (self.table["sw"] - self.swcr) / (1 - self.swcr - self.sorw)
         # Normalize for krow:
-        self.table["son"] = (1 - self.table.sw - self.sorw) / (
-            1 - self.sorw - self.swcr
+        self.table["son"] = (1 - self.table["sw"] - self.sorw) / (
+            1 - self.sorw - self.swl
         )
 
         # Different normalization for Sw used for capillary pressure
-        self.table["swnpc"] = (self.table.sw - swirr) / (1 - swirr)
+        self.table["swnpc"] = (self.table["sw"] - swirr) / (1 - swirr)
+
+        if _sgcr is None:
+            self.swcomment = "-- swirr=%g swl=%g swcr=%g sorw=%g\n" % (
+                self.swirr,
+                self.swl,
+                self.swcr,
+                self.sorw,
+            )
+        else:
+            # When _sgcr is defined, this object is in use by GasWater
+            self.swcomment = "-- swirr=%g swl=%g swcr=%g sgrw=%g sgcr=%g\n" % (
+                self.swirr,
+                self.swl,
+                self.swcr,
+                self.sorw,
+                self.sgcr,
+            )
 
-        self.swcomment = "-- swirr=%g swl=%g swcr=%g sorw=%g\n" % (
-            self.swirr,
-            self.swl,
-            self.swcr,
-            self.sorw,
-        )
         self.krwcomment = ""
         self.krowcomment = ""
         self.pccomment = ""
 
-        logger.info(
+        logger.debug(
             "Initialized WaterOil with %s saturation points", str(len(self.table))
         )
 
@@ -197,8 +225,6 @@ class WaterOil(object):
             sorw (float): Explicit sorw. If None, it will be estimated from
                 the numbers in krw (or krow)
         """
-        from scipy.interpolate import PchipInterpolator, interp1d
-
         # Avoid having to deal with multi-indices:
         if len(dframe.index.names) > 1:
             logger.warning(
@@ -354,15 +380,13 @@ class WaterOil(object):
             self.pccomment = "-- pc from tabular input" + pccomment + "\n"
 
     def add_corey_water(self, nw=2, krwend=1, krwmax=None):
-        """ Add krw data through the Corey parametrization
+        """Add krw data through the Corey parametrization
 
         A column named 'krw' will be added. If it exists, it will
         be replaced.
 
-        It is assumed that there are no sw points between
-        sw=1-sorw and sw=1, which should give linear
-        interpolations in simulators. The corey parameter
-        applies up to 1-sorw.
+        The Corey model applies for sw < 1 - sorw. For higher
+        water saturations, krw is linear between krwend and krwmax.
 
         krwmax will be ignored if sorw is close to zero
 
@@ -370,7 +394,6 @@ class WaterOil(object):
             nw (float): Corey parameter for water.
             krwend (float): value of krw at 1 - sorw.
             krwmax (float): maximal value at Sw=1. Default 1
-
         """
         assert epsilon < nw < MAX_EXPONENT
         if krwmax:
@@ -378,7 +401,7 @@ class WaterOil(object):
         else:
             assert 0 < krwend <= 1.0
 
-        self.table["krw"] = krwend * self.table.swn ** nw
+        self.table["krw"] = krwend * self.table["swn"] ** nw
 
         self.set_endpoints_linearpart_krw(krwend, krwmax)
 
@@ -417,7 +440,7 @@ class WaterOil(object):
             self.table.iloc[linear_section_indices[-1]]["krw"] = krwmax
         else:
             if krwmax is not None:
-                logger.warning("krwmax ignored when sorw is zero")
+                logger.info("krwmax ignored when sorw is zero")
 
         # If the linear section is longer than two rows, do linear
         # interpolation inside for krw:
@@ -431,53 +454,34 @@ class WaterOil(object):
             self.table.loc[:, "krw"] = interp_krw.values
 
         # Left linear section is all zero:
-        self.table.loc[self.table.sw < self.swcr, "krw"] = 0
+        self.table.loc[self.table["sw"] < self.swcr, "krw"] = 0
 
     def set_endpoints_linearpart_krow(self, kroend, kromax=None):
         """Set linear parts of krow outside endpoints
 
-        Curve will be linear in [swl, swcr] (from kromax to kroend)
-        and zero in [1 - sorw, 1]
+        Curve will be zero in [1 - sorw, 1].
 
         This function is used by add_corey_water(), and perhaps by other
         utility functions. It should not be necessary for end-users.
 
         Args:
             kroend (float): value of kro at swcr
-            kromax (float): maximal value of kro at sw=swl. Default 1
         """
-
-        # Linear curve between swl and swcr (left part):
-        self.table.loc[self.table["son"] > 1.0 + epsilon, "krow"] = np.nan
-        if self.swcr < self.swl + epsilon:
-            if kromax and kromax < 1.0:
-                # Only warn if non-default value
-                logger.warning("kromax ignored when swcr is close to swl")
-            self.table.loc[self.table["sw"] <= self.swl + epsilon, "krow"] = kroend
-        else:
-            if not kromax:
-                kromax = 1
-            self.table.loc[self.table["sw"] <= self.swl + epsilon, "krow"] = kromax
-            interp_krow = (
-                self.table[["sw", "krow"]]
-                .set_index("sw")
-                .interpolate(method="index")["krow"]
-            )
-            self.table.loc[:, "krow"] = interp_krow.values
-
-        # Avoid machine accuracy problems around swcr (monotonicity):
-        self.table.loc[self.table["krow"] > kromax, "krow"] = kromax
+        if kromax is not None:
+            logger.error("kromax is DEPRECATED, ignored")
 
         # Set to zero above sorw:
         self.table.loc[self.table["sw"] > 1 - self.sorw, "krow"] = 0
 
+        # Floating point issues can cause this to have become
+        # slightly bigger than krowend.
+        self.table.loc[self.table["krow"] > kroend, "krow"] = kroend
+
     def add_LET_water(self, l=2, e=2, t=2, krwend=1, krwmax=None):
         """Add krw data through LET parametrization
 
-        It is assumed that there are no sw points between
-        sw=1-sorw and sw=1, which should give linear
-        interpolations in simulators. The LET parameters
-        apply up to 1-sorw.
+        The LET model applies for sw < 1 - sgrw. For higher
+        water saturations, krw is linear between krwend and krwmax.
 
         krwmax will be ignored if sorw is close to zero.
 
@@ -488,6 +492,9 @@ class WaterOil(object):
             krwend (float): value of krw at 1 - sorw
             krwmax (float): maximal value at Sw=1. Default 1
         """
+        # Similar code in gasoil.add_LET_gas, but readability is
+        # better by having them separate.
+        # pylint: disable=duplicate-code
         assert epsilon < l < MAX_EXPONENT
         assert epsilon < e < MAX_EXPONENT
         assert epsilon < t < MAX_EXPONENT
@@ -498,8 +505,8 @@ class WaterOil(object):
 
         self.table["krw"] = (
             krwend
-            * self.table.swn ** l
-            / ((self.table.swn ** l) + e * (1 - self.table.swn) ** t)
+            * self.table["swn"] ** l
+            / ((self.table["swn"] ** l) + e * (1 - self.table["swn"]) ** t)
         )
         # This equation is undefined for t a float and swn=1, set explicitly:
         self.table.loc[np.isclose(self.table["swn"], 1.0), "krw"] = krwend
@@ -520,14 +527,11 @@ class WaterOil(object):
         """
         Add kro data through LET parametrization
 
-        kromax will be ignored if swcr is close to swl.
-
         Args:
             l (float): LET parameter
             e (float): LET parameter
             t (float): LET parameter
-            kroend (float): value of kro at swcr
-            kromax (float): maximal value of kro at sw=swl. Default 1
+            kroend (float): value of kro at swl
 
         Returns:
             None (modifies object)
@@ -535,31 +539,28 @@ class WaterOil(object):
         assert epsilon < l < MAX_EXPONENT
         assert epsilon < e < MAX_EXPONENT
         assert epsilon < t < MAX_EXPONENT
-        if kromax:
-            assert 0 < kroend <= kromax <= 1.0
-        else:
-            assert 0 < kroend <= 1.0
+        assert 0 < kroend <= 1.0
+
+        if kromax is not None:
+            logger.error("kromax is DEPRECATED, ignored")
 
         self.table["krow"] = (
             kroend
-            * self.table.son ** l
-            / ((self.table.son ** l) + e * (1 - self.table.son) ** t)
+            * self.table["son"] ** l
+            / ((self.table["son"] ** l) + e * (1 - self.table["son"]) ** t)
         )
         # This equation is undefined for t a float and son=1, set explicitly:
         self.table.loc[np.isclose(self.table["son"], 1.0), "krow"] = kroend
 
-        self.table.loc[self.table.sw >= (1 - self.sorw), "krow"] = 0
+        self.table.loc[self.table["sw"] >= (1 - self.sorw), "krow"] = 0
 
-        self.set_endpoints_linearpart_krow(kroend, kromax)
+        self.set_endpoints_linearpart_krow(kroend)
 
-        if not kromax:
-            kromax = 1
-        self.krowcomment = "-- LET krow, l=%g, e=%g, t=%g, kroend=%g, kromax=%g\n" % (
+        self.krowcomment = "-- LET krow, l=%g, e=%g, t=%g, kroend=%g\n" % (
             l,
             e,
             t,
             kroend,
-            kromax,
         )
 
     def add_corey_oil(self, now=2, kroend=1, kromax=None):
@@ -569,33 +570,24 @@ class WaterOil(object):
 
         Curve is linear between swl and swcr, zero above 1 - sorw.
 
-        kromax will be ignored if swcr is close to swl.
-
         Args:
             now (float): Corey exponent
             kroend (float): kro value at swcr
-            kromax (float): kro value at swl
         Returns:
             None (modifies object)
         """
         assert epsilon < now < MAX_EXPONENT
-        if kromax:
-            assert 0 < kroend <= kromax <= 1.0
-        else:
-            assert 0 < kroend <= 1.0
+        assert 0 < kroend <= 1.0
 
-        self.table["krow"] = kroend * self.table.son ** now
-        self.table.loc[self.table.sw >= (1 - self.sorw), "krow"] = 0
+        if kromax is not None:
+            logger.error("kromax is DEPRECATED, ignored")
 
-        self.set_endpoints_linearpart_krow(kroend, kromax)
+        self.table["krow"] = kroend * self.table["son"] ** now
+        self.table.loc[self.table["sw"] >= (1 - self.sorw), "krow"] = 0
 
-        if not kromax:
-            kromax = 1
-        self.krowcomment = "-- Corey krow, now=%g, kroend=%g, kromax=%g\n" % (
-            now,
-            kroend,
-            kromax,
-        )
+        self.set_endpoints_linearpart_krow(kroend)
+
+        self.krowcomment = "-- Corey krow, now=%g, kroend=%g\n" % (now, kroend,)
 
     def add_simple_J(self, a=5, b=-1.5, poro_ref=0.25, perm_ref=100, drho=300, g=9.81):
         """Add capillary pressure function from a simplified J-function
@@ -654,10 +646,10 @@ class WaterOil(object):
         # swnpc is a normalized saturation, but normalized with
         # respect to swirr, not to swl (the swirr here is sometimes
         # called 'swirra' - asymptotic swirr)
-        self.table["J"] = a * self.table.swnpc ** b
-        self.table["H"] = self.table.J * math.sqrt(poro_ref / perm_ref)
+        self.table["J"] = a * self.table["swnpc"] ** b
+        self.table["H"] = self.table["J"] * math.sqrt(poro_ref / perm_ref)
         # Scale drho and g from SI units to g/cc and m/s²100
-        self.table["pc"] = self.table.H * drho / 1000 * g / 100.0
+        self.table["pc"] = self.table["H"] * drho / 1000 * g / 100.0
         self.pccomment = (
             "-- Simplified J-function for Pc; rms version, in bar\n--   "
             + "a=%g, b=%g, poro_ref=%g, perm_ref=%g mD, drho=%g kg/m^3, g=%g m/s^2\n"
@@ -806,7 +798,7 @@ class WaterOil(object):
         swirr and sorw. Only use different values here if you know
         what you are doing.
 
-        Modifies or adds self.table.pc if succesful.
+        Modifies or adds self.table["pc"] if succesful.
         Returns false if error occured.
 
         """  # noqa
@@ -852,20 +844,20 @@ class WaterOil(object):
 
         # swnpc is generated upon object initialization, but overwritten
         # here to most likely the same values.
-        self.table["swnpc"] = (self.table.sw - swr) / (1 - swr)
+        self.table["swnpc"] = (self.table["sw"] - swr) / (1 - swr)
 
         # sonpc is almost like 'son', but swl is not used here:
-        self.table["sonpc"] = (1 - self.table.sw - sor) / (1 - sor)
+        self.table["sonpc"] = (1 - self.table["sw"] - sor) / (1 - sor)
 
         # The Skjæveland correlation
-        self.table.loc[self.table.sw < 1 - sor, "pc"] = cw / (
-            self.table.swnpc ** aw
-        ) + co / (self.table.sonpc ** ao)
+        self.table.loc[self.table["sw"] < 1 - sor, "pc"] = cw / (
+            self.table["swnpc"] ** aw
+        ) + co / (self.table["sonpc"] ** ao)
 
         # From 1-sor, the pc is not defined. We want to extrapolate constantly,
         # but with a twist as Eclipse does not non-monotone capillary pressure:
-        self.table["pc"].fillna(value=self.table.pc.min(), inplace=True)
-        nanrows = self.table.sw > 1 - sor - epsilon
+        self.table["pc"].fillna(value=self.table["pc"].min(), inplace=True)
+        nanrows = self.table["sw"] > 1 - sor - epsilon
         self.table.loc[nanrows, "pc"] = (
             self.table.loc[nanrows, "pc"] - self.table.loc[nanrows, "sw"]
         )  # Just deduct sw to make it monotone..
@@ -923,7 +915,7 @@ class WaterOil(object):
         assert Pcmin <= Pct <= Pcmax
 
         # Normalized water saturation including sorw
-        self.table["swnpco"] = (self.table.sw - self.swirr) / (
+        self.table["swnpco"] = (self.table["sw"] - self.swirr) / (
             1 - self.sorw - self.swirr
         )
 
@@ -983,7 +975,7 @@ class WaterOil(object):
             self.table, xcol="sw", ycol=curve, side="right"
         )
 
-    def estimate_swcr(self, curve="krow"):
+    def estimate_swcr(self, curve="krw"):
         """Estimate swcr of the current krw data.
 
         swcr is estimated by searching for a linear part in krw upwards from sw=swl.
@@ -1011,27 +1003,12 @@ class WaterOil(object):
 
         Accuracy of this crosspoint depends on the resolution chosen
         when initializing the saturation range
-       """
 
-        # Make a copy for calculations
-        tmp = pd.DataFrame(self.table[["sw", "krw", "krow"]])
-        tmp.loc[:, "krwminuskrow"] = tmp["krw"] - tmp["krow"]
-
-        # Add a zero value for the difference column, and interpolate
-        # the sg column to the zero value
-        zerodf = pd.DataFrame(index=[len(tmp)], data={"krwminuskrow": 0.0})
-        tmp = pd.concat([tmp, zerodf], sort=True)
-        # When Pandas is upgraded for all users:
-        # tmp = pd.concat([tmp, zerodf], sort=True)
-        tmp.set_index("krwminuskrow", inplace=True)
-
-        if tmp.index.isnull().any():
-            logger.warning("Could not compute crosspoint. Bug?")
-            return -1
-
-        tmp.interpolate(method="slinear", inplace=True)
-
-        return tmp[np.isclose(tmp.index, 0.0)].sw.values[0]
+        Returns:
+            float: the water saturation where krw == krow, for relperm
+                linearly interpolated in water saturation.
+        """
+        return utils.crosspoint(self.table, "sw", "krw", "krow")
 
     def selfcheck(self, mode="SWOF"):
         """Check validities of the data in the table.
@@ -1089,7 +1066,7 @@ class WaterOil(object):
                 error = True
         if error:
             return False
-        logger.info("WaterOil object is checked to be valid")
+        logger.debug("WaterOil object is checked to be valid")
         return True
 
     def SWOF(self, header=True, dataincommentrow=True):
@@ -1109,6 +1086,7 @@ class WaterOil(object):
                 and emit the SWOF yourself). Default True
             dataincommentrow (bool): Wheter metadata should
                 be printed. Defualt True
+
         """
         if not self.fast and not self.selfcheck():
             # selfcheck failed and has issued an error message
@@ -1145,8 +1123,30 @@ class WaterOil(object):
         string += "/\n"  # Empty line at the end
         return string
 
-    def SWFN(self, header=True, dataincommentrow=True):
-        """Return a SWFN keyword with data to Eclipse"""
+    def SWFN(
+        self, header=True, dataincommentrow=True, swcomment=None, crosspointcomment=None
+    ):
+        """Return a SWFN keyword with data to Eclipse
+
+        The columns sw, krw and pc are outputted and formatted
+        accordingly.
+
+        Meta-information for the tabulated data are printed
+        as Eclipse comments.
+
+        Args:
+            header: boolean for whether the SWFN string should be emitted.
+                If you have multiple satnums, you should have True only
+                for the first (or False for all, and emit the SWFN yourself).
+                Defaults to True.
+            dataincommentrow: boolean for wheter metadata should be printed,
+                defaults to True.
+            swcomment (str): String to be used for swcomment, overrides what
+                this object can provide. Used by GasWater
+            crosspointcomment (str): String to be used for crosspoint comment
+                string, overrides what this object can provide. Used by GasWater.
+                If None, it will be computed, use empty string to avoid.
+        """
         if not self.selfcheck(mode="SWFN"):
             # selfcheck will print errors/warnings
             return ""
@@ -1159,10 +1159,16 @@ class WaterOil(object):
         string += utils.comment_formatter(self.tag)
         string += "-- pyscal: " + str(pyscal.__version__) + "\n"
         if dataincommentrow:
-            string += self.swcomment
+            if swcomment is not None:
+                string += swcomment
+            else:
+                string += self.swcomment
             string += self.krwcomment
-            if "krow" in self.table.columns and not self.fast:
-                string += "-- krw = krow @ sw=%1.5f\n" % self.crosspoint()
+            if crosspointcomment is None:
+                if "krow" in self.table.columns and not self.fast:
+                    string += "-- krw = krow @ sw=%1.5f\n" % self.crosspoint()
+            else:
+                string += crosspointcomment
             string += self.pccomment
         width = 10
         string += (
