@@ -193,6 +193,29 @@ class WaterOilGas(object):
         string += "/\n"
         return string
 
+    @property
+    def swirr(self):
+        return self.wateroil.swirr
+
+    @property
+    def swl(self):
+        return self.wateroil.swl
+
+    @property
+    def sorg(self):
+        return self.gasoil.sorg
+
+    @property
+    def sorw(self):
+        return self.wateroil.sorw
+
+    @property
+    def tag(self):
+        if self.wateroil.tag == self.gasoil.tag:
+            return self.wateroil.tag
+        else:
+            return self.wateroil.tag + " " + self.gasoil.tag
+
     def threephaseconsistency(self):
         """Perform consistency checks on produced curves, similar
         to what Eclipse does at startup.
@@ -227,7 +250,11 @@ class WaterOilGas(object):
         if not np.isclose(
             self.wateroil.table["krow"].max(), self.gasoil.table["krog"].max()
         ):
-            logger.warning("Eclipse will fail, max(krow) is not equal to max(krog)")
+            logger.warning(
+                "Eclipse will fail, max(krow)=%g is not equal to max(krog)=%g",
+                self.wateroil.table["krow"].max(),
+                self.gasoil.table["krog"].max(),
+            )
             wog_is_ok = False
 
         # 2: Inconsistent end points in saturation table 1 the maximum
